!     ####################
      MODULE MODD_SURF_ATM_TURB_n
!     ######################
!
!!****  *MODD_SURF_ATM_TURB - declaration of surface parameters linked to atmospheric turbulence
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!      D. Degrauwe
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       09/2017
!


!*       0.   DECLARATIONS
!             ------------
!

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SURF_ATM_TURB_t
  ! variables from YOMPHY
  LOGICAL         :: LCOEFKTKE=.FALSE.   ! default value as safety: other values are only used if LCOEFKTKE=.TRUE.
  LOGICAL         :: LCOEFK_F1
  CHARACTER(LEN=20) :: CGTURS
  
  ! variables from YOMPHY0
	REAL(KIND=JPRB) :: XNUPTKE
	REAL(KIND=JPRB) :: XC3TKEFREE
	REAL(KIND=JPRB) :: XETKE_OLAM
	REAL(KIND=JPRB) :: XEFB_UR
	REAL(KIND=JPRB) :: XEFB_AZ0
	REAL(KIND=JPRB) :: XREFB_1
	REAL(KIND=JPRB) :: XREFB_2
	REAL(KIND=JPRB) :: XREFB_3
  
  ! variables from YOMQNSE
  REAL(KIND=JPRB) :: XRQNSE_S0
  REAL(KIND=JPRB) :: XRQNSE_GU1
  REAL(KIND=JPRB) :: XRQNSE_GU2
  REAL(KIND=JPRB) :: XRQNSE_GS1
  REAL(KIND=JPRB) :: XRQNSE_GS2
  REAL(KIND=JPRB) :: XRQNSE_GS3
  REAL(KIND=JPRB) :: XRQNSE_GS4
  
  ! variables from YOMLOUIS
	REAL(KIND=JPRB) :: XRLOUIS_S0
	REAL(KIND=JPRB) :: XPLOUIS_S0
	REAL(KIND=JPRB) :: XRLOUIS_GU1
	REAL(KIND=JPRB) :: XRLOUIS_GU2
	REAL(KIND=JPRB) :: XPLOUIS_GU1
	REAL(KIND=JPRB) :: XPLOUIS_GU2
	REAL(KIND=JPRB) :: XRLOUIS_GS1
	REAL(KIND=JPRB) :: XRLOUIS_GS2
	REAL(KIND=JPRB) :: XRLOUIS_GS3
	REAL(KIND=JPRB) :: XRLOUIS_GS4
	REAL(KIND=JPRB) :: XPLOUIS_GS1
	REAL(KIND=JPRB) :: XPLOUIS_GS2
	REAL(KIND=JPRB) :: XPLOUIS_GS3
	REAL(KIND=JPRB) :: XPLOUIS_GS4
  
END TYPE SURF_ATM_TURB_t

!! TYPE(SURF_ATM_TURB_t) :: YRSURF_ATM_TURB        ! preference for passing by argument instead of with global variable.

CONTAINS

SUBROUTINE SURF_ATM_TURB_SET(YDATMTURB, &
 & LDLCOEFKTKE, LDLCOEFK_F1, CDCGTURS, &
 & PNUPTKE, PC3TKEFREE, PETKE_OLAM, PEFB_UR, PEFB_AZ0, PREFB_1, PREFB_2, PREFB_3, &
 & PRQNSE_S0, PRQNSE_GU1, PRQNSE_GU2, PRQNSE_GS1, PRQNSE_GS2, PRQNSE_GS3, PRQNSE_GS4, &
 & PRLOUIS_S0, PPLOUIS_S0, PRLOUIS_GU1, PRLOUIS_GU2, PPLOUIS_GU1, PPLOUIS_GU2, &
 & PRLOUIS_GS1, PRLOUIS_GS2, PRLOUIS_GS3, PRLOUIS_GS4, &
 & PPLOUIS_GS1, PPLOUIS_GS2, PPLOUIS_GS3, PPLOUIS_GS4 )

! arguments
TYPE(SURF_ATM_TURB_t), INTENT(INOUT) :: YDATMTURB
LOGICAL        , INTENT(IN) :: LDLCOEFKTKE
LOGICAL        , INTENT(IN) :: LDLCOEFK_F1
CHARACTER(LEN=20), INTENT(IN) :: CDCGTURS
REAL(KIND=JPRB), INTENT(IN) :: PNUPTKE
REAL(KIND=JPRB), INTENT(IN) :: PC3TKEFREE
REAL(KIND=JPRB), INTENT(IN) :: PETKE_OLAM
REAL(KIND=JPRB), INTENT(IN) :: PEFB_UR
REAL(KIND=JPRB), INTENT(IN) :: PEFB_AZ0
REAL(KIND=JPRB), INTENT(IN) :: PREFB_1
REAL(KIND=JPRB), INTENT(IN) :: PREFB_2
REAL(KIND=JPRB), INTENT(IN) :: PREFB_3
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_S0
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GU1
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GU2
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GS1
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GS2
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GS3
REAL(KIND=JPRB), INTENT(IN) :: PRQNSE_GS4
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_S0
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_S0
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GU1
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GU2
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GU1
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GU2
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GS1
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GS2
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GS3
REAL(KIND=JPRB), INTENT(IN) :: PRLOUIS_GS4
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GS1
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GS2
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GS3
REAL(KIND=JPRB), INTENT(IN) :: PPLOUIS_GS4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_TURB_N:SURF_ATM_TURB_SET",0,ZHOOK_HANDLE)

YDATMTURB%LCOEFKTKE  = LDLCOEFKTKE
YDATMTURB%LCOEFK_F1  = LDLCOEFK_F1
YDATMTURB%CGTURS     = CDCGTURS
YDATMTURB%XNUPTKE     = PNUPTKE
YDATMTURB%XC3TKEFREE  = PC3TKEFREE
YDATMTURB%XETKE_OLAM  = PETKE_OLAM
YDATMTURB%XEFB_UR     = PEFB_UR
YDATMTURB%XEFB_AZ0    = PEFB_AZ0
YDATMTURB%XREFB_1     = PREFB_1
YDATMTURB%XREFB_2     = PREFB_2
YDATMTURB%XREFB_3     = PREFB_3
YDATMTURB%XRQNSE_S0   = PRQNSE_S0
YDATMTURB%XRQNSE_GU1  = PRQNSE_GU1
YDATMTURB%XRQNSE_GU2  = PRQNSE_GU2
YDATMTURB%XRQNSE_GS1  = PRQNSE_GS1
YDATMTURB%XRQNSE_GS2  = PRQNSE_GS2
YDATMTURB%XRQNSE_GS3  = PRQNSE_GS3
YDATMTURB%XRQNSE_GS4  = PRQNSE_GS4
YDATMTURB%XRLOUIS_S0  = PRLOUIS_S0
YDATMTURB%XPLOUIS_S0  = PPLOUIS_S0
YDATMTURB%XRLOUIS_GU1 = PRLOUIS_GU1
YDATMTURB%XRLOUIS_GU2 = PRLOUIS_GU2
YDATMTURB%XPLOUIS_GU1 = PPLOUIS_GU1
YDATMTURB%XPLOUIS_GU2 = PPLOUIS_GU2
YDATMTURB%XRLOUIS_GS1 = PRLOUIS_GS1
YDATMTURB%XRLOUIS_GS2 = PRLOUIS_GS2
YDATMTURB%XRLOUIS_GS3 = PRLOUIS_GS3
YDATMTURB%XRLOUIS_GS4 = PRLOUIS_GS4
YDATMTURB%XPLOUIS_GS1 = PPLOUIS_GS1
YDATMTURB%XPLOUIS_GS2 = PPLOUIS_GS2
YDATMTURB%XPLOUIS_GS3 = PPLOUIS_GS3
YDATMTURB%XPLOUIS_GS4 = PPLOUIS_GS4

IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_TURB_N:SURF_ATM_TURB_SET",1,ZHOOK_HANDLE)

END SUBROUTINE SURF_ATM_TURB_SET

END MODULE MODD_SURF_ATM_TURB_n

